enum MatchingMode {
Check,
Bind,
Assign,
}Expand description
We need to do pattern matching in two phases: One to check if the pattern matches, and one to fill the bindings of patterns. This is necessary to prevent double moves and similar problems. For example:
ⓘ
struct X;
match (X, 3) {
(b, 2) | (b, 3) => {},
_ => {}
}If we do everything in one pass, we will move X to the first b, then we see that the second field of tuple
doesn’t match and we should move the X to the second b (which here is the same thing, but doesn’t need to be) and
it might even doesn’t match the second pattern and we may want to not move X at all.
Variants
Check
Check that if this pattern matches
Bind
Assume that this pattern matches, fill bindings
Assign
Assume that this pattern matches, assign to existing variables.